#pragma once

enum class AccelerometerMma8452qRegister : uint8_t {
    GLOBAL_ENABLE = 1,
    DATA_ENABLE,
    DATA_CONFIG,
    DATA_VALUE,
    MOVEMENT_ENABLE,
    MOVEMENT_CONFIG,
    MOVEMENT_VALUE,
    ORIENTATION_ENABLE,
    ORIENTATION_CONFIG,
    ORIENTATION_VALUE,
    PULSE_ENABLE,
    PULSE_CONFIG,
    PULSE_STATUS,
    SHAKE_ENABLE,
    SHAKE_CONFIG,
    SHAKE_STATUS,
    PACKED_ACC_DATA= 0x12
};
