class Math:
    OPERATION_ADD= 1
    OPERATION_MULTIPLY= 2
    OPERATION_DIVIDE= 3
    OPERATION_MODULUS= 4
    OPERATION_EXPONENT= 5
    OPERATION_SQRT= 6
    OPERATION_LSHIFT= 7
    OPERATION_RSHIFT= 8
    OPERATION_SUBTRACT= 9
    OPERATION_ABS_VALUE= 10
    OPERATION_CONSTANT= 11

class Comparator:
    OPERATION_EQ = 0
    OPERATION_NEQ = 1
    OPERATION_LT = 2
    OPERATION_LTE = 3
    OPERATION_GT = 4
    OPERATION_GTE = 5

    MODE_ABSOLUTE = 0
    MODE_REFERENCE = 1
    MODE_ZONE = 2
    MODE_BINARY = 3

class Threshold:
    MODE_ABSOLUTE= 0
    MODE_BINARY= 1

class Time:
    MODE_ABSOLUTE= 0
    MODE_DIFFERENTIAL= 1

class Passthrough:
    MODE_ALL= 0
    MODE_CONDITIONAL= 1
    MODE_COUNT= 2

class Delta:
    MODE_ABSOLUTE= 0
    MODE_DIFFERENTIAL= 1
    MODE_BINARY= 2

class Pulse:
    OUTPUT_WIDTH= 0
    OUTPUT_AREA= 1
    OUTPUT_PEAK= 2
    OUTPUT_ON_DETECTION= 3
