#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
version.py
-----------

:copyright: 2016-11-28 by hbldh <henrik.blidh@nedomkull.com>

"""

from __future__ import division
from __future__ import print_function
from __future__ import absolute_import

__version__ = '0.7.2'
version = __version__  # backwards compatibility name
